// CDTV IR Remote
// By Paul Rickards
// rickards@gmail.com
// June 5, 2011
// Decodes original Nintendo NES game pad and converts to infrared (IR)
// signals appropriate for an Amiga CDTV, or in this case a Chameleon
// cartridge (which uses the same style).
// IR signals are modulated at 40 Khz.
// Tested in Arduino 0018

// Based on code from
// https://github.com/shirriff/Arduino-IRremote
// Pins 2 and 11 might not be usable due to PWM timer2 usage
// Added "sendCDTV" function to IRremote library

// Based on CDTV IR description from Peter whom made
// the IR receiver code in the Chameleon cartridge.
// http://tech.groups.yahoo.com/group/chameleon_64/message/630

// CDTV IR messages are 24 bits in length
// 12 bits + the complement of those 12 bits
// Joystick directions start with the most-significant (leftmost) bit set to '1'
// Four directions plus two buttons (A+B). Active pressing = 1, otherwise = 0
// 1000 ABUD LR00
// This allows more than one button to be pressed simultaneously (i.e. left + button A)
// by setting the appropriate bits

#include <IRremote.h>

IRsend irsend;

int irled = 3;  // this is for reference only, IRremote library only uses pin 3
int latch = 8;  // set the latch pin for NES controller
int clock = 9;  // set the clock pin for NES controller
int datin = 10; // set the data in pin for NES controller
int ledpin = 13;   // set the led status pin, on when transmitting IR

int up, down, left, right, a, b, select, start;
unsigned long out;

void setup()
{
  // Setup NES controller
  pinMode(latch, OUTPUT);
  pinMode(clock, OUTPUT);
  pinMode(datin, INPUT);
  pinMode(ledpin, OUTPUT);
  digitalWrite(latch, HIGH);
  digitalWrite(clock, HIGH);
}

void loop() {
  NEScontrollerRead();
  sendIR();
}

void sendIR() {
  out = 0x8007FF00; // "centered" joystick (nothing pressed), last 8 bits are padding, not used
  // Modify outbound message appropriately based on what's being pressed
  // bitSet and bitClear start from the least significant bit (rightmost)
  // Adding 8 skips the padding added above (we only need 24 bits of the 32 bit unsigned long)
  if (a) { bitSet(out, 19+8); bitClear(out, 7+8); }
  if (b) { bitSet(out, 18+8); bitClear(out, 6+8); }
  if (up) { bitSet(out, 17+8); bitClear(out, 5+8); }
  if (down) { bitSet(out, 16+8); bitClear(out, 4+8); }
  if (left) { bitSet(out, 15+8); bitClear(out, 3+8); }
  if (right) { bitSet(out, 14+8); bitClear(out, 2+8); }

  if (up || down || left || right || a || b || select || start) {
    irsend.sendCDTV(out, 24);
    digitalWrite(ledpin, 1);
  }
  else { digitalWrite(ledpin, 0); }
}

void NEScontrollerRead() {
  digitalWrite(latch, LOW);
  digitalWrite(clock, LOW);

  digitalWrite(latch, HIGH);
  delayMicroseconds(2);
  digitalWrite(latch, LOW);

  // button A
  a = !digitalRead(datin);
  NESCycleClock();
  
  // button B
  b = !digitalRead(datin);
  NESCycleClock();
  
  // button Select
  select = !digitalRead(datin);
  NESCycleClock();
  
  // button Start
  start = !digitalRead(datin);
  NESCycleClock();
  
  // button Up
  up = !digitalRead(datin);
  NESCycleClock();
  
  // button Down
  down = !digitalRead(datin);
  NESCycleClock();
  
  // button Left
  left = !digitalRead(datin);
  NESCycleClock();
  
  // button Right
  right = !digitalRead(datin);
  NESCycleClock();
}

void NESCycleClock() {
  delayMicroseconds(4);
  digitalWrite(clock, LOW);
  digitalWrite(clock, HIGH);
  delayMicroseconds(2);
}

